@@ -52,16 +52,19 @@
 #include "eog-collection-marshal.h"
 #include "eog-full-screen.h"
 #include "eog-info-view.h"
+#include "eog-transform.h"
 
 enum {
 	PROP_WINDOW_TITLE,
 	PROP_WINDOW_STATUS,
+	PROP_IMAGE_PROGRESS,
 	PROP_LAST
 };
 
 static const gchar *property_name[] = {
 	"window/title", 
-	"window/status"
+	"window/status",
+	"image/progress"
 };
 
 enum {
@@ -95,6 +98,9 @@ struct _EogCollectionViewPrivate {
 	gboolean                need_update_prop[PROP_LAST];
 
 	EogImage                *displayed_image;
+	guint                   progress_handler_id;
+
+	gboolean                has_zoomable_frame;
 };
 
 
@@ -164,25 +170,225 @@ verb_ImageNext_cb (BonoboUIComponent *uic,
 	eog_wrap_list_select_right (EOG_WRAP_LIST (view->priv->wraplist));
 }
 
+static void 
+verb_Undo_cb (BonoboUIComponent *uic, gpointer data, const char *name)
+{
+	GList *images;
+	GList *it;
+	EogCollectionView *view;
+
+	view = EOG_COLLECTION_VIEW (data);
+
+	images = eog_wrap_list_get_selected_images (EOG_WRAP_LIST (view->priv->wraplist));	
+
+	for (it = images; it != NULL; it = it->next) {
+		EogImage *image = EOG_IMAGE (it->data);
+		eog_image_undo (image);
+	}
+}
+
+typedef struct {
+	EogCollectionView *view;
+	float max_progress;
+	int counter;
+} ProgressData;
+
+static void
+image_progress_cb (EogImage *image, float progress, gpointer data)
+{
+	ProgressData *pd;
+	float total;
+	BonoboArg *arg;
+	EogCollectionViewPrivate *priv;
+
+	if (EOG_IS_COLLECTION_VIEW (data)) {
+		pd = NULL;
+		priv = EOG_COLLECTION_VIEW (data)->priv;
+	}
+	else {
+		pd = (ProgressData*) data;
+		priv = pd->view->priv;
+	}
+
+	if (!pd) {
+		total = progress;
+	}
+	else {
+		total = ((float) pd->counter + progress) / pd->max_progress;
+	}
+
+	arg = bonobo_arg_new (BONOBO_ARG_FLOAT);
+	BONOBO_ARG_SET_FLOAT (arg, total);
+
+	bonobo_event_source_notify_listeners (priv->property_bag->es,
+					      "image/progress",
+					      arg, NULL);
+	bonobo_arg_release (arg);
+}
+
+static void 
+apply_transformation (EogCollectionView *view, EogTransform *trans)
+{
+	GList *images;
+	GList *it;
+	gint id;
+	ProgressData *data;
+
+	images = eog_wrap_list_get_selected_images (EOG_WRAP_LIST (view->priv->wraplist));	
+	data = g_new0 (ProgressData, 1);
+	data->view = view;
+	data->max_progress = g_list_length (images);
+	data->counter = 0;
+
+	for (it = images; it != NULL; it = it->next) {
+		EogImage *image = EOG_IMAGE (it->data);
+
+		id = g_signal_connect (G_OBJECT (image), "progress", (GCallback) image_progress_cb, data);
+
+		eog_image_transform (image, trans);
+
+		g_signal_handler_disconnect (image, id);
+		data->counter++;
+	}
+
+	g_free (data);
+	g_object_unref (trans);
+}
+
+static void
+verb_FlipHorizontal_cb (BonoboUIComponent *uic, gpointer data, const char *name)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (data));
+
+	apply_transformation (EOG_COLLECTION_VIEW (data), eog_transform_flip_new (EOG_TRANSFORM_FLIP_HORIZONTAL));
+}
+
+static void
+verb_FlipVertical_cb (BonoboUIComponent *uic, gpointer data, const char *name)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (data));
+
+	apply_transformation (EOG_COLLECTION_VIEW (data), eog_transform_flip_new (EOG_TRANSFORM_FLIP_VERTICAL));
+}
+
+static void
+verb_Rotate90ccw_cb (BonoboUIComponent *uic, gpointer data, const char *name)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (data));
+
+	apply_transformation (EOG_COLLECTION_VIEW (data), eog_transform_rotate_new (270));
+}
+
+static void
+verb_Rotate90cw_cb (BonoboUIComponent *uic, gpointer data, const char *name)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (data));
+
+	apply_transformation (EOG_COLLECTION_VIEW (data), eog_transform_rotate_new (90));
+}
+
+static void
+verb_Rotate180_cb (BonoboUIComponent *uic, gpointer data, const char *name)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (data));
+
+	apply_transformation (EOG_COLLECTION_VIEW (data), eog_transform_rotate_new (180));
+}
+
+static void
+verb_ZoomIn_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname)
+{
+	EogCollectionView *view;
+
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (user_data));
+
+	view = EOG_COLLECTION_VIEW (user_data);
+
+	eog_scroll_view_zoom_in (EOG_SCROLL_VIEW (view->priv->scroll_view), FALSE);
+}
+
+static void
+verb_ZoomOut_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname)
+{
+	EogCollectionView *view;
+
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (user_data));
+
+	view = EOG_COLLECTION_VIEW (user_data);
+
+	eog_scroll_view_zoom_out (EOG_SCROLL_VIEW (view->priv->scroll_view), FALSE);
+}
+
+static void
+verb_ZoomToDefault_cb (BonoboUIComponent *uic, gpointer user_data,
+		       const char *cname)
+{
+	EogCollectionView *view;
+
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (user_data));
+
+	view = EOG_COLLECTION_VIEW (user_data);
+
+	eog_scroll_view_set_zoom (EOG_SCROLL_VIEW (view->priv->scroll_view), 1.0);
+}
+
+static void
+verb_ZoomToFit_cb (BonoboUIComponent *uic, gpointer user_data,
+		   const char *cname)
+{
+	EogCollectionView *view;
+
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (user_data));
+
+	view = EOG_COLLECTION_VIEW (user_data);
+
+	eog_scroll_view_zoom_fit (EOG_SCROLL_VIEW (view->priv->scroll_view));
+}
+
+static BonoboUIVerb eog_zoom_verbs[] = {
+	BONOBO_UI_VERB ("ZoomIn",        verb_ZoomIn_cb),
+	BONOBO_UI_VERB ("ZoomOut",       verb_ZoomOut_cb),
+	BONOBO_UI_VERB ("ZoomToDefault", verb_ZoomToDefault_cb),
+	BONOBO_UI_VERB ("ZoomToFit",     verb_ZoomToFit_cb),
+	BONOBO_UI_VERB_END
+};
+
 static BonoboUIVerb collection_verbs[] = {
 	BONOBO_UI_VERB ("SlideShow", verb_SlideShow_cb),
 	BONOBO_UI_VERB ("ImageNext", verb_ImageNext_cb),
 	BONOBO_UI_VERB ("ImagePrev", verb_ImagePrev_cb),
+	BONOBO_UI_VERB ("FlipHorizontal",verb_FlipHorizontal_cb),
+	BONOBO_UI_VERB ("FlipVertical",  verb_FlipVertical_cb),
+	BONOBO_UI_VERB ("Rotate90cw",    verb_Rotate90cw_cb),
+	BONOBO_UI_VERB ("Rotate90ccw",   verb_Rotate90ccw_cb),
+	BONOBO_UI_VERB ("Rotate180",     verb_Rotate180_cb),
+	BONOBO_UI_VERB ("Undo",          verb_Undo_cb),
 	BONOBO_UI_VERB_END
 };
 
 static void
 eog_collection_view_create_ui (EogCollectionView *view)
 {
+	EogCollectionViewPrivate *priv;
+
 	g_return_if_fail (EOG_IS_COLLECTION_VIEW (view));
 
+	priv = view->priv;
+
 	/* Set up the UI from an XML file. */
-        bonobo_ui_util_set_ui (view->priv->uic, DATADIR,
+        bonobo_ui_util_set_ui (priv->uic, DATADIR,
 			       "eog-collection-view-ui.xml", "EogCollectionView", NULL);
 
-	bonobo_ui_component_add_verb_list_with_data (view->priv->uic,
+	bonobo_ui_component_add_verb_list_with_data (priv->uic,
 						     collection_verbs,
 						     view);
+
+	if (!priv->has_zoomable_frame) {
+		bonobo_ui_util_set_ui (priv->uic, DATADIR, "eog-image-view-ctrl-ui.xml", "EogCollectionView", NULL);
+		
+		bonobo_ui_component_add_verb_list_with_data (priv->uic, eog_zoom_verbs,
+							     view);
+	}
 }
 
 static void
@@ -483,17 +689,22 @@ handle_selection_changed (EogWrapList *list, EogCollectionView *view)
 
 	priv = view->priv;
 
-	if (priv->displayed_image != NULL) {
-		eog_image_free_mem (priv->displayed_image);
-		g_object_unref (priv->displayed_image);
-		priv->displayed_image = NULL;
+	if (priv->displayed_image != image) {
+		if (priv->displayed_image != NULL) {
+			g_signal_handler_disconnect (priv->displayed_image, priv->progress_handler_id);
+
+			eog_image_free_mem (priv->displayed_image);
+			g_object_unref (priv->displayed_image);
+			priv->displayed_image = NULL;
+		}
+		
+		priv->progress_handler_id = g_signal_connect (image, "progress", G_CALLBACK (image_progress_cb), view);
+		eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->scroll_view), image);
+		eog_info_view_set_image (EOG_INFO_VIEW (priv->info_view), image);
+		
+		priv->displayed_image = image;
 	}
-
-	eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->scroll_view), image);
-	eog_info_view_set_image (EOG_INFO_VIEW (priv->info_view), image);
-
-	priv->displayed_image = image;
-	
+		
 	update_status_text (view);
 }
 
@@ -540,6 +751,68 @@ layout_changed_cb (GConfClient *client, guint cnxn_id,
 				      layout);
 }
 
+static void
+zoomable_set_frame_cb (BonoboZoomable *zoomable, EogCollectionView *view)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (view));
+
+	view->priv->has_zoomable_frame = TRUE;
+}
+
+static void
+zoomable_set_zoom_level_cb (BonoboZoomable *zoomable, CORBA_float new_zoom_level,
+			    EogCollectionView *view)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (view));
+
+	eog_scroll_view_set_zoom (EOG_SCROLL_VIEW (view->priv->scroll_view), new_zoom_level);
+}
+
+static void
+zoomable_zoom_in_cb (BonoboZoomable *zoomable, EogCollectionView *view)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (view));
+
+	eog_scroll_view_zoom_in (EOG_SCROLL_VIEW (view->priv->scroll_view), FALSE);
+}
+
+static void
+zoomable_zoom_out_cb (BonoboZoomable *zoomable, EogCollectionView *view)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (view));
+
+	eog_scroll_view_zoom_out (EOG_SCROLL_VIEW (view->priv->scroll_view), FALSE);
+}
+
+static void
+zoomable_zoom_to_fit_cb (BonoboZoomable *zoomable, EogCollectionView *view)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (view));
+
+	eog_scroll_view_zoom_fit (EOG_SCROLL_VIEW (view->priv->scroll_view));
+}
+
+static void
+zoomable_zoom_to_default_cb (BonoboZoomable *zoomable, EogCollectionView *view)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (view));
+
+	eog_scroll_view_set_zoom (EOG_SCROLL_VIEW (view->priv->scroll_view), 1.0);
+}
+
+static void
+view_zoom_changed_cb (GtkWidget *widget, double zoom, gpointer data)
+{
+	EogCollectionView *view;
+	EogCollectionViewPrivate *priv;
+
+	view = EOG_COLLECTION_VIEW (data);
+	priv = view->priv;
+
+	/* inform zoom interface listeners */
+	bonobo_zoomable_report_zoom_level_changed (priv->zoomable, zoom, NULL);
+}
+
 /* read configuration */
 static void
 init_gconf_defaults (EogCollectionView *view)
@@ -635,6 +908,10 @@ create_user_interface (EogCollectionView *list_view)
 	/* the image view for the full size image */
  	priv->scroll_view = eog_scroll_view_new ();
 	g_object_set (G_OBJECT (priv->scroll_view), "height_request", 250, NULL);
+	g_signal_connect (G_OBJECT (priv->scroll_view),
+			  "zoom_changed",
+			  (GCallback) view_zoom_changed_cb,
+			  list_view);
 	frame = gtk_widget_new (GTK_TYPE_FRAME, "shadow-type", GTK_SHADOW_IN, NULL);
 	gtk_container_add (GTK_CONTAINER (frame), priv->scroll_view);
 
@@ -692,6 +969,7 @@ eog_collection_view_construct (EogCollectionView *list_view)
 	EogCollectionViewPrivate *priv = NULL;
 	BonoboControl *control;
 	GtkWidget *root;
+	BonoboZoomable *zoomable;
 
 	g_return_val_if_fail (list_view != NULL, NULL);
 	g_return_val_if_fail (EOG_IS_COLLECTION_VIEW (list_view), NULL);
@@ -713,6 +991,46 @@ eog_collection_view_construct (EogCollectionView *list_view)
 				     BONOBO_OBJECT (control));
 
 
+	/* Interface Bonobo::Zoomable */
+	zoomable = bonobo_zoomable_new ();
+	priv->zoomable = zoomable;
+
+	g_signal_connect (G_OBJECT (zoomable),
+			  "set_frame",
+			  G_CALLBACK (zoomable_set_frame_cb),
+			  list_view);
+	g_signal_connect (G_OBJECT (zoomable),
+			  "set_zoom_level",
+			  G_CALLBACK (zoomable_set_zoom_level_cb),
+			  list_view);
+	g_signal_connect (G_OBJECT (zoomable),
+			  "zoom_in",
+			  G_CALLBACK (zoomable_zoom_in_cb),
+			  list_view);
+	g_signal_connect (G_OBJECT (zoomable),
+			  "zoom_out",
+			  G_CALLBACK (zoomable_zoom_out_cb),
+			  list_view);
+	g_signal_connect (G_OBJECT (zoomable),
+			  "zoom_to_fit",
+			  G_CALLBACK (zoomable_zoom_to_fit_cb),
+			  list_view);
+	g_signal_connect (G_OBJECT (zoomable),
+			  "zoom_to_default",
+			  G_CALLBACK (zoomable_zoom_to_default_cb),
+			  list_view);
+
+	bonobo_zoomable_set_parameters (zoomable,
+					1.0,   /* current */
+					0.01,  /* min */
+					20.0,  /* max */
+					TRUE,
+					TRUE);
+
+	bonobo_object_add_interface (BONOBO_OBJECT (list_view),
+				     BONOBO_OBJECT (zoomable));
+
+
 	/* Property Bag */
 	priv->property_bag = bonobo_property_bag_new (eog_collection_view_get_prop,
 						      eog_collection_view_set_prop,
@@ -723,6 +1041,9 @@ eog_collection_view_construct (EogCollectionView *list_view)
 	bonobo_property_bag_add (priv->property_bag, property_name[PROP_WINDOW_STATUS], PROP_WINDOW_STATUS,
 				 BONOBO_ARG_STRING, NULL, _("Status Text"),
 				 BONOBO_PROPERTY_READABLE);
+	bonobo_property_bag_add (priv->property_bag, "image/progress", PROP_IMAGE_PROGRESS,
+				 BONOBO_ARG_FLOAT, NULL, _("Progress of Image Loading"),
+				 BONOBO_PROPERTY_READABLE);
 	bonobo_control_set_properties (BONOBO_CONTROL (control), 
 				       BONOBO_OBJREF (priv->property_bag),
 				       NULL);
